import django.test
from django.contrib.auth.models import User
from ..models import Token
from .base import make_expired


class TokenModelCase(django.test.TestCase):
    """Test case for token model"""

    def setUp(self):
        self._user = User.objects.create_user('test', 'test@test.test', 'test')

    def test_should_generate_token_on_first_save(self):
        """Test that token should be generated on first save"""
        token = Token.objects.create(owner=self._user)
        self.assertIsNotNone(token.value)

    def test_should_not_regenerate_token(self):
        """Test that token should not regenerate token on second save"""
        token = Token.objects.create(owner=self._user)
        value = token.value
        token.save()
        self.assertEqual(token.value, value)

    def test_get_expired_tokens(self):
        """Test get expired tokens"""
        for _ in range(5):
            Token.objects.create(owner=self._user)
        expired = [
            make_expired(Token.objects.create(owner=self._user))
            for _ in range(3)
        ]
        self.assertItemsEqual(
            Token.objects.get_expired(), expired,
        )

    def test_has_valid_tokens(self):
        """Test user has valid tokens"""
        token = Token.objects.create(owner=self._user)
        self.assertTrue(
            Token.objects.has_valid(self._user, token.value),
        )

    def test_has_no_valid_tokens_without_token(self):
        """Test has no valid tokens without tokens"""
        self.assertFalse(
            Token.objects.has_valid(self._user, 'token'),
        )

    def test_has_no_valid_token_when_expired(self):
        """Test has not valid tokens when expired"""
        token = Token.objects.create(owner=self._user)
        make_expired(token)
        self.assertFalse(
            Token.objects.has_valid(self._user, token.value),
        )

    def test_has_valid_token_for_view(self):
        """Test has valid token for view"""
        token = Token.objects.create(owner=self._user, for_view='test')
        self.assertTrue(
            Token.objects.has_valid(self._user, token.value, 'test'),
        )



from datetime import timedelta
from django.conf import settings


ANON_COOKIE = getattr(settings, 'ANON_COOKIE', 'anoncsrf')
ANON_TIMEOUT = getattr(settings, 'ANON_TIMEOUT', 60 * 60 * 2)  # 2 hours.
ANON_ALWAYS = getattr(settings, 'ANON_ALWAYS', False)
PREFIX = 'sessioncsrf:'

CSRF_TOKEN_LIFETIME = getattr(
    settings, 'CSRF_TOKEN_LIFETIME', timedelta(days=1),
)



from jinja2 import nodes
from coffin.template.defaulttags import CsrfTokenExtension
from coffin import template
from ..utils import get_token_for_request


register = template.Library()


@register.tag
class PerViewCSRFExtension(CsrfTokenExtension):
    """Per-view csrf token for jinja2"""

    tags = {'per_view_csrf'}

    def parse(self, parser):
        """Parse tokens from template"""
        lineno = parser.stream.next().lineno
        view_name = parser.parse_expression()
        return nodes.Output([
            self.call_method('_render', [
                nodes.Name('csrf_token', 'load'),
                view_name,
                nodes.Name('request', 'load'),
            ]),
        ]).set_lineno(lineno)

    def _render(self, csrf_token, view_name, request):
        """Render csrf token"""
        token = get_token_for_request(request, view_name)
        if token is not None:
            csrf_token = token.value
        return super(PerViewCSRFExtension, self)._render(csrf_token)



from django.core.cache import cache
from django.middleware import csrf as django_csrf
from django.utils import crypto
from django.utils.cache import patch_vary_headers
from .models import Token
from .utils import prep_key
from . import conf


class CsrfMiddleware(object):

    # csrf_processing_done prevents checking CSRF more than once. That could
    # happen if the requires_csrf_token decorator is used.
    def _accept(self, request):
        request.csrf_processing_done = True

    def _reject(self, request, reason):
        return django_csrf._get_failure_view()(request, reason)

    def _has_valid_csrf(self, request):
        """Is request has valid csrf token"""
        if 'csrf_token' not in request.session:
            return False
        else:
            return Token.objects.has_valid(
                request.user,
                request.session['csrf_token']
            )

    def process_request(self, request):
        """
        Add a CSRF token to the session for logged-in users.

        The token is available at request.csrf_token.
        """
        if hasattr(request, 'csrf_token'):
            return
        if request.user.is_authenticated():
            if self._has_valid_csrf(request):
                request.csrf_token = request.session['csrf_token']
            else:
                token = Token.objects.create(owner=request.user).value
                request.csrf_token = request.session['csrf_token'] = token
        else:
            key = None
            token = ''
            if conf.ANON_COOKIE in request.COOKIES:
                key = request.COOKIES[conf.ANON_COOKIE]
                token = cache.get(prep_key(key), '')
            if conf.ANON_ALWAYS:
                if not key:
                    key = django_csrf._get_new_csrf_key()
                if not token:
                    token = django_csrf._get_new_csrf_key()
                request._anon_csrf_key = key
                cache.set(prep_key(key), token, conf.ANON_TIMEOUT)
            request.csrf_token = token

    def _check_per_view_csrf(self, request, view, user_token):
        """Check per view csrf token"""
        view_full_name = '{}.{}'.format(
            view.__module__,
            view.__name__,
        )
        return Token.objects.has_valid(
            request.user, user_token, view_full_name)

    def _need_per_view_csrf(self, request, view):
        """Is view need per-view csrf token"""
        if (
            view and request.user.is_authenticated()
            and hasattr(view, 'per_view_csrf')
        ):
            return True
        else:
            return False

    def process_view(self, request, view_func, *args, **kwargs):
        """Check the CSRF token if this is a POST."""
        if getattr(request, 'csrf_processing_done', False):
            return

        # Allow @csrf_exempt views.
        if getattr(view_func, 'csrf_exempt', False):
            return

        if (getattr(view_func, 'anonymous_csrf_exempt', False)
            and not request.user.is_authenticated()):
            return

        # Bail if this is a safe method.
        if request.method in ('GET', 'HEAD', 'OPTIONS', 'TRACE'):
            return self._accept(request)

        # The test client uses this to get around CSRF processing.
        if getattr(request, '_dont_enforce_csrf_checks', False):
            return self._accept(request)

        # Try to get the token from the POST and fall back to looking at the
        # X-CSRFTOKEN header.
        user_token = request.POST.get('csrfmiddlewaretoken', '')
        if user_token == '':
            user_token = request.META.get('HTTP_X_CSRFTOKEN', '')

        if self._need_per_view_csrf(request, view_func):
            if self._check_per_view_csrf(request, view_func, user_token):
                return self._accept(request)
            else:
                return self._reject(request, django_csrf.REASON_BAD_TOKEN)

        request_token = getattr(request, 'csrf_token', '')
        # Check that both strings aren't empty and then check for a match.
        if not ((user_token or request_token)
                and crypto.constant_time_compare(user_token, request_token))\
                or (request.user.is_authenticated()
                    and not Token.objects.has_valid(request.user, request_token)):
            reason = django_csrf.REASON_BAD_TOKEN
            django_csrf.logger.warning(
                'Forbidden (%s): %s' % (reason, request.path),
                extra=dict(status_code=403, request=request))
            return self._reject(request, reason)
        else:
            return self._accept(request)

    def process_response(self, request, response):
        if hasattr(request, '_anon_csrf_key'):
            # Set or reset the cache and cookie timeouts.
            response.set_cookie(conf.ANON_COOKIE, request._anon_csrf_key,
                                max_age=conf.ANON_TIMEOUT, httponly=True,
                                secure=request.is_secure())
            patch_vary_headers(response, ['Cookie'])
        return response



from django.utils.decorators import method_decorator
from .decorators import per_view_csrf


class PerViewCsrfMixin(object):
    """Per view csrf mixin"""

    @method_decorator(per_view_csrf)
    def dispatch(self, *args, **kwargs):
        return super(PerViewCsrfMixin, self).dispatch(*args, **kwargs)



# This overrides django.core.context_processors.csrf to dump our csrf_token
# into the template context.
def context_processor(request):
    # Django warns about an empty token unless you call it NOTPROVIDED.
    return {'csrf_token': getattr(request, 'csrf_token', 'NOTPROVIDED')}



import mock
import django.test
from django.core.cache import cache
from django.contrib.sessions.models import Session
from django.contrib.auth.models import User
from ..utils import prep_key
from ..decorators import per_view_csrf
from .. import conf
from .base import ClientHandler


class TestAnonymousCsrf(django.test.TestCase):
    urls = 'session_csrf.tests'

    def setUp(self):
        self.token = 'a' * 32
        self.rf = django.test.RequestFactory()
        User.objects.create_user('jbalogh', 'j@moz.com', 'password')
        self.client.handler = ClientHandler(enforce_csrf_checks=True)
        self.save_ANON_ALWAYS = conf.ANON_ALWAYS
        conf.ANON_ALWAYS = False

    def tearDown(self):
        conf.ANON_ALWAYS = self.save_ANON_ALWAYS

    def login(self):
        assert self.client.login(username='jbalogh', password='password')

    def test_authenticated_request(self):
        # Nothing special happens, nothing breaks.
        # Find the CSRF token in the session.
        self.login()
        response = self.client.get('/anon')
        sessionid = response.cookies['sessionid'].value
        session = Session.objects.get(session_key=sessionid)
        token = session.get_decoded()['csrf_token']

        response = self.client.post('/anon', HTTP_X_CSRFTOKEN=token)
        self.assertEqual(response.status_code, 200)

    def test_unauthenticated_request(self):
        # We get a 403 since we're not sending a token.
        response = self.client.post('/anon')
        self.assertEqual(response.status_code, 403)

    def test_no_anon_cookie(self):
        # We don't get an anon cookie on non-@anonymous_csrf views.
        response = self.client.get('/')
        self.assertEqual(response.cookies, {})

    def test_new_anon_token_on_request(self):
        # A new anon user gets a key+token on the request and response.
        response = self.client.get('/anon')
        # Get the key from the cookie and find the token in the cache.
        key = response.cookies[conf.ANON_COOKIE].value
        self.assertEqual(response._request.csrf_token, cache.get(prep_key(key)))

    def test_existing_anon_cookie_on_request(self):
        # We reuse an existing anon cookie key+token.
        response = self.client.get('/anon')
        key = response.cookies[conf.ANON_COOKIE].value
        # Now check that subsequent requests use that cookie.
        response = self.client.get('/anon')
        self.assertEqual(response.cookies[conf.ANON_COOKIE].value, key)
        self.assertEqual(response._request.csrf_token, cache.get(prep_key(key)))

    def test_new_anon_token_on_response(self):
        # The anon cookie is sent and we vary on Cookie.
        response = self.client.get('/anon')
        self.assertIn(conf.ANON_COOKIE, response.cookies)
        self.assertEqual(response['Vary'], 'Cookie')

    def test_existing_anon_token_on_response(self):
        # The anon cookie is sent and we vary on Cookie, reusing the old value.
        response = self.client.get('/anon')
        key = response.cookies[conf.ANON_COOKIE].value

        response = self.client.get('/anon')
        self.assertEqual(response.cookies[conf.ANON_COOKIE].value, key)
        self.assertIn(conf.ANON_COOKIE, response.cookies)
        self.assertEqual(response['Vary'], 'Cookie')

    def test_anon_csrf_logout(self):
        # Beware of views that logout the user.
        self.login()
        response = self.client.get('/logout')
        self.assertEqual(response.status_code, 200)

    def test_existing_anon_cookie_not_in_cache(self):
        response = self.client.get('/anon')
        self.assertEqual(len(response._request.csrf_token), 32)

        # Clear cache and make sure we still get a token
        cache.clear()
        response = self.client.get('/anon')
        self.assertEqual(len(response._request.csrf_token), 32)

    def test_anonymous_csrf_exempt(self):
        response = self.client.post('/no-anon-csrf')
        self.assertEqual(response.status_code, 200)

        self.login()
        response = self.client.post('/no-anon-csrf')
        self.assertEqual(response.status_code, 403)


class TestAnonAlways(django.test.TestCase):
    # Repeats some tests with ANON_ALWAYS = True
    urls = 'session_csrf.tests'

    def setUp(self):
        self.token = 'a' * 32
        self.rf = django.test.RequestFactory()
        User.objects.create_user('jbalogh', 'j@moz.com', 'password')
        self.client.handler = ClientHandler(enforce_csrf_checks=True)
        self.save_ANON_ALWAYS = conf.ANON_ALWAYS
        conf.ANON_ALWAYS = True

    def tearDown(self):
        conf.ANON_ALWAYS = self.save_ANON_ALWAYS

    def login(self):
        assert self.client.login(username='jbalogh', password='password')

    def test_csrftoken_unauthenticated(self):
        # request.csrf_token is set for anonymous users
        # when ANON_ALWAYS is enabled.
        response = self.client.get('/', follow=True)
        # The CSRF token is a 32-character MD5 string.
        self.assertEqual(len(response._request.csrf_token), 32)

    def test_authenticated_request(self):
        # Nothing special happens, nothing breaks.
        # Find the CSRF token in the session.
        self.login()
        response = self.client.get('/', follow=True)
        sessionid = response.cookies['sessionid'].value
        session = Session.objects.get(session_key=sessionid)
        token = session.get_decoded()['csrf_token']

        response = self.client.post('/', follow=True, HTTP_X_CSRFTOKEN=token)
        self.assertEqual(response.status_code, 200)

    def test_unauthenticated_request(self):
        # We get a 403 since we're not sending a token.
        response = self.client.post('/')
        self.assertEqual(response.status_code, 403)

    def test_new_anon_token_on_request(self):
        # A new anon user gets a key+token on the request and response.
        response = self.client.get('/')
        # Get the key from the cookie and find the token in the cache.
        key = response.cookies[conf.ANON_COOKIE].value
        self.assertEqual(response._request.csrf_token, cache.get(prep_key(key)))

    def test_existing_anon_cookie_on_request(self):
        # We reuse an existing anon cookie key+token.
        response = self.client.get('/')
        key = response.cookies[conf.ANON_COOKIE].value

        # Now check that subsequent requests use that cookie.
        response = self.client.get('/')
        self.assertEqual(response.cookies[conf.ANON_COOKIE].value, key)
        self.assertEqual(response._request.csrf_token, cache.get(prep_key(key)))
        self.assertEqual(response['Vary'], 'Cookie')

    def test_anon_csrf_logout(self):
        # Beware of views that logout the user.
        self.login()
        response = self.client.get('/logout')
        self.assertEqual(response.status_code, 200)

    def test_existing_anon_cookie_not_in_cache(self):
        response = self.client.get('/')
        self.assertEqual(len(response._request.csrf_token), 32)

        # Clear cache and make sure we still get a token
        cache.clear()
        response = self.client.get('/')
        self.assertEqual(len(response._request.csrf_token), 32)

    def test_massive_anon_cookie(self):
        # if the key + PREFIX + setting prefix is greater than 250
        # memcache will cry and you get a warning if you use LocMemCache
        junk = 'x' * 300
        with mock.patch('warnings.warn') as warner:
            response = self.client.get('/', HTTP_COOKIE='anoncsrf=%s' % junk)
            self.assertEqual(response.status_code, 200)
            self.assertEqual(warner.call_count, 0)

    def test_surprising_characters(self):
        c = 'anoncsrf="|dir; multidb_pin_writes=y; sessionid="gAJ9cQFVC'
        with mock.patch('warnings.warn') as warner:
            response = self.client.get('/', HTTP_COOKIE=c)
            self.assertEqual(response.status_code, 200)
            self.assertEqual(warner.call_count, 0)


class PerViewCsrfCase(django.test.TestCase):
    """per_view_csrf test case"""

    def test_should_add_flag(self):
        """Decorator should add per_view_csrf_flag"""
        view = per_view_csrf(lambda: None)
        self.assertTrue(view.per_view_csrf)



from django.template.defaulttags import CsrfTokenNode
from django import template
from ..utils import save_token, get_token_for_request


register = template.Library()


@register.simple_tag(takes_context=True)
def per_view_csrf(context, view_name):
    """Register per view csrf token. Not pure!"""
    with save_token(context):
        token = get_token_for_request(context['request'], view_name)
        if token is not None:
            context['csrf_token'] = token.value
        return CsrfTokenNode().render(context)
per_view_csrf.is_safe = True



from datetime import datetime, timedelta
import django.test.client
try:
    from django.conf.urls import patterns
except ImportError:
    from django.conf.urls.defaults import patterns
from django import http
from django.contrib.auth import logout
from django.core import signals
from django.core.handlers.wsgi import WSGIRequest
from django.db import close_connection
from ..decorators import anonymous_csrf, anonymous_csrf_exempt, per_view_csrf
from .. import conf


@per_view_csrf
def per_view(request):
    return http.HttpResponse()


urlpatterns = patterns('',
    ('^$', lambda r: http.HttpResponse()),
    ('^anon$', anonymous_csrf(lambda r: http.HttpResponse())),
    ('^no-anon-csrf$', anonymous_csrf_exempt(lambda r: http.HttpResponse())),
    ('^logout$', anonymous_csrf(lambda r: logout(r) or http.HttpResponse())),
    ('^per-view$', per_view)
)


def make_expired(token):
    """Make token expired"""
    token.created =\
        datetime.now() - conf.CSRF_TOKEN_LIFETIME - timedelta(days=1)
    token.save()
    return token


class ClientHandler(django.test.client.ClientHandler):
    """
    Handler that stores the real request object on the response.

    Almost all the code comes from the parent class.
    """

    def __call__(self, environ):
        # Set up middleware if needed. We couldn't do this earlier, because
        # settings weren't available.
        if self._request_middleware is None:
            self.load_middleware()

        signals.request_started.send(sender=self.__class__)
        try:
            request = WSGIRequest(environ)
            # sneaky little hack so that we can easily get round
            # CsrfViewMiddleware.  This makes life easier, and is probably
            # required for backwards compatibility with external tests against
            # admin views.
            request._dont_enforce_csrf_checks = not self.enforce_csrf_checks
            response = self.get_response(request)
        finally:
            signals.request_finished.disconnect(close_connection)
            signals.request_finished.send(sender=self.__class__)
            signals.request_finished.connect(close_connection)

        # Store the request object.
        response._request = request
        return response



import functools
from django.core.cache import cache
from django.middleware import csrf as django_csrf
from django.utils.cache import patch_vary_headers
from .utils import prep_key
from . import conf


def anonymous_csrf(f):
    """Decorator that assigns a CSRF token to an anonymous user."""
    @functools.wraps(f)
    def wrapper(request, *args, **kw):
        use_anon_cookie = not (request.user.is_authenticated() or conf.ANON_ALWAYS)
        if use_anon_cookie:
            if conf.ANON_COOKIE in request.COOKIES:
                key = request.COOKIES[conf.ANON_COOKIE]
                token = cache.get(prep_key(key)) or django_csrf._get_new_csrf_key()
            else:
                key = django_csrf._get_new_csrf_key()
                token = django_csrf._get_new_csrf_key()
            cache.set(prep_key(key), token, conf.ANON_TIMEOUT)
            request.csrf_token = token
        response = f(request, *args, **kw)
        if use_anon_cookie:
            # Set or reset the cache and cookie timeouts.
            response.set_cookie(conf.ANON_COOKIE, key, max_age=conf.ANON_TIMEOUT,
                                httponly=True, secure=request.is_secure())
            patch_vary_headers(response, ['Cookie'])
        return response
    return wrapper


def anonymous_csrf_exempt(f):
    """Like @csrf_exempt but only for anonymous requests."""
    f.anonymous_csrf_exempt = True
    return f


def per_view_csrf(fn):
    """Require per view csrf"""
    fn.per_view_csrf = True
    return fn



from mock import MagicMock
from django.contrib.auth.models import User
from django.test import TestCase
from ..models import Token
from ..utils import save_token, get_token_for_request


class TestUtils(TestCase):
    """Test case for utils"""

    def test_should_not_modify_token(self):
        """Test should not modify token"""
        context = {'csrf_token': 'test'}
        with save_token(context):
            context['csrf_token'] = 'wtf'
        self.assertEqual(context['csrf_token'], 'test')

    def test_get_token_for_authenticated(self):
        """Test get token for authenticated"""
        user = User.objects.create()
        user.is_authenticated = lambda: True
        token = get_token_for_request(MagicMock(user=user), 'test')
        self.assertIsInstance(token, Token)

    def test_get_none_for_anonymous(self):
        """Test get None for anonymous"""
        request = MagicMock()
        request.user.is_authenticated.return_value = False
        token = get_token_for_request(request, 'test')
        self.assertIsNone(token)



from mock import MagicMock
from django.contrib.auth.models import User
from django.test import TestCase
from ..models import Token
from ..templatetags.session_csrf import per_view_csrf


class PerViewCsrfCase(TestCase):
    """per_view_csrf templatetag case"""

    def test_should_generate_csrf_token_when_authenticated(self):
        """Test template tag should generate csrf when authenticated"""
        request = MagicMock(user=User.objects.create())
        request.user.is_authenticated = lambda: True
        self.assertNotEqual(per_view_csrf({
            'request': request,
        }, 'test'), '')
        self.assertTrue(Token.objects.filter(
            owner=request.user,
            for_view='test',
        ).exists())

    def test_should_fallback_to_default_csrf_when_not_authenticated(self):
        """Test should fallback to default csrf when not authenticated"""
        request = MagicMock()
        request.user.is_authenticated.return_value = False
        self.assertNotEqual(per_view_csrf({
            'request': request,
            'csrf_token': 'test',
        }, 'test'), '')



import mock
import django.test
from django.contrib.auth.middleware import AuthenticationMiddleware
from django.contrib.auth.models import User
from django.contrib.sessions.middleware import SessionMiddleware
from django.core.cache import cache
from django.template import context
from ..models import Token
from ..middlewares import CsrfMiddleware
from ..utils import prep_key
from .. import conf
from .base import ClientHandler, make_expired


class TestCsrfToken(django.test.TestCase):

    def setUp(self):
        self.client.handler = ClientHandler()
        User.objects.create_user('jbalogh', 'j@moz.com', 'password')
        self.save_ANON_ALWAYS = conf.ANON_ALWAYS
        conf.ANON_ALWAYS = False

    def tearDown(self):
        conf.ANON_ALWAYS = self.save_ANON_ALWAYS

    def login(self):
        assert self.client.login(username='jbalogh', password='password')

    def test_csrftoken_unauthenticated(self):
        # request.csrf_token is '' for anonymous users.
        response = self.client.get('/', follow=True)
        self.assertEqual(response._request.csrf_token, '')

    def test_csrftoken_authenticated(self):
        # request.csrf_token is a random non-empty string for authed users.
        self.login()
        response = self.client.get('/', follow=True)
        # The CSRF token is a 32-character MD5 string.
        self.assertEqual(len(response._request.csrf_token), 32)

    def test_csrftoken_new_session(self):
        # The csrf_token is added to request.session the first time.
        self.login()
        response = self.client.get('/', follow=True)
        # The CSRF token is a 32-character MD5 string.
        token = response._request.session['csrf_token']
        self.assertEqual(len(token), 32)
        self.assertEqual(token, response._request.csrf_token)

    def test_csrftoken_existing_session(self):
        # The csrf_token in request.session is reused on subsequent requests.
        self.login()
        r1 = self.client.get('/', follow=True)
        token = r1._request.session['csrf_token']

        r2 = self.client.get('/', follow=True)
        self.assertEqual(r1._request.csrf_token, r2._request.csrf_token)
        self.assertEqual(token, r2._request.csrf_token)


class TestCsrfMiddleware(django.test.TestCase):

    def setUp(self):
        self.token = 'a' * 32
        self.rf = django.test.RequestFactory()
        self.mw = CsrfMiddleware()
        self._user = User.objects.create()

    def process_view(self, request, view=None):
        request.session = {}
        return self.mw.process_view(request, view, None, None)

    def test_anon_token_from_cookie(self):
        rf = django.test.RequestFactory()
        rf.cookies[conf.ANON_COOKIE] = self.token
        cache.set(prep_key(self.token), 'woo')
        request = rf.get('/')
        SessionMiddleware().process_request(request)
        AuthenticationMiddleware().process_request(request)
        self.mw.process_request(request)
        self.assertEqual(request.csrf_token, 'woo')

    def test_set_csrftoken_once(self):
        # Make sure process_request only sets request.csrf_token once.
        request = self.rf.get('/')
        request.csrf_token = 'woo'
        self.mw.process_request(request)
        self.assertEqual(request.csrf_token, 'woo')

    def test_reject_view(self):
        # Check that the reject view returns a 403.
        response = self.process_view(self.rf.post('/'))
        self.assertEqual(response.status_code, 403)

    def test_csrf_exempt(self):
        # Make sure @csrf_exempt still works.
        view = type("", (), {'csrf_exempt': True})()
        self.assertEqual(self.process_view(self.rf.post('/'), view), None)

    def test_safe_whitelist(self):
        # CSRF should not get checked on these methods.
        self.assertEqual(self.process_view(self.rf.get('/')), None)
        self.assertEqual(self.process_view(self.rf.head('/')), None)
        self.assertEqual(self.process_view(self.rf.options('/')), None)

    def test_unsafe_methods(self):
        self.assertEqual(self.process_view(self.rf.post('/')).status_code,
                         403)
        self.assertEqual(self.process_view(self.rf.put('/')).status_code,
                         403)
        self.assertEqual(self.process_view(self.rf.delete('/')).status_code,
                         403)

    def test_csrfmiddlewaretoken(self):
        # The user token should be found in POST['csrfmiddlewaretoken'].
        request = self.rf.post('/', {'csrfmiddlewaretoken': self.token})
        self.assertEqual(self.process_view(request).status_code, 403)

        request.csrf_token = self.token
        self.assertEqual(self.process_view(request), None)

    def test_x_csrftoken(self):
        # The user token can be found in the X-CSRFTOKEN header.
        request = self.rf.post('/', HTTP_X_CSRFTOKEN=self.token)
        self.assertEqual(self.process_view(request).status_code, 403)

        request.csrf_token = self.token
        self.assertEqual(self.process_view(request), None)

    def test_require_request_token_or_user_token(self):
        # Blank request and user tokens raise an error on POST.
        request = self.rf.post('/', HTTP_X_CSRFTOKEN='')
        request.csrf_token = ''
        self.assertEqual(self.process_view(request).status_code, 403)

    def test_token_no_match(self):
        # A 403 is returned when the tokens don't match.
        request = self.rf.post('/', HTTP_X_CSRFTOKEN='woo')
        request.csrf_token = ''
        self.assertEqual(self.process_view(request).status_code, 403)

    def test_csrf_token_context_processor(self):
        # Our CSRF token should be available in the template context.
        request = mock.Mock()
        request.csrf_token = self.token
        request.groups = []
        ctx = {}
        for processor in context.get_standard_processors():
            ctx.update(processor(request))
        self.assertEqual(ctx['csrf_token'], self.token)

    def _authenticated_request(self, token=None, **kwargs):
        """Create mocked request object for authenticated user"""
        self._user.is_authenticated = lambda: True
        if token is None:
            token = Token.objects.create(owner=self._user).value
        return mock.MagicMock(
            csrf_token=token,
            user=self._user,
            POST={},
            META={'HTTP_X_CSRFTOKEN': token},
            csrf_processing_done=False,
            _dont_enforce_csrf_checks=False,
            **kwargs)

    def test_reject_for_wrong_token_if_authenticated(self):
        """Test reject for wrong token if authenticated"""
        request = self._authenticated_request('wrong')
        self.assertIsNotNone(self.process_view(request))

    def test_reject_when_token_expired(self):
        """Test reject when csrf token expired"""
        token = make_expired(Token.objects.create(owner=self._user))
        request = self._authenticated_request(token.value)
        self.assertIsNotNone(self.process_view(request))

    def test_accept_when_token_is_ok(self):
        """Test accept when token is ok"""
        request = self._authenticated_request()
        self.assertIsNone(self.process_view(request))

    def test_renew_csrf_token_on_request_if_expired(self):
        """Test renew csrf token on request if expired"""
        token = make_expired(Token.objects.create(owner=self._user))
        request = self._authenticated_request(token.value, session={
            'csrf_token': token.value,
        })
        del request.csrf_token
        self.mw.process_request(request)
        self.assertNotEqual(token.value, request.csrf_token)

    def test_not_change_csrf_token_on_request_if_valid(self):
        """Test not change csrf token on request if valid"""
        request = self._authenticated_request()
        token = request.csrf_token
        request.session = {
            'csrf_token': token,
        }
        del request.csrf_token
        self.mw.process_request(request)
        self.assertEqual(token, request.csrf_token)

    def test_add_csrf_token_on_request(self):
        """Test add csrf token on request"""
        request = self._authenticated_request()
        del request.csrf_token
        self.mw.process_request(request)
        self.assertIsNotNone(request.csrf_token)


class TestPerViewCsrf(django.test.TestCase):
    """Per view csrf test case"""

    def setUp(self):
        self.user = User.objects.create_user('test', 'test@test.test', 'test')
        self.client.handler = ClientHandler()
        self.client.login(username='test', password='test')

    def _get_token(self):
        return Token.objects.create(
            owner=self.user,
            for_view="session_csrf.tests.base.per_view",
        )

    def test_ok_with_correct_per_view_csrf(self):
        """Test response is ok with correct per-view csrf"""
        response = self.client.post('/per-view', {
            'csrfmiddlewaretoken': self._get_token().value,
        })
        self.assertEqual(response.status_code, 200)

    def test_not_ok_with_expired_csrf_token(self):
        """Test not ok with expired csrf token"""
        token = make_expired(self._get_token())
        response = self.client.post('/per-view', {
            'csrfmiddlewaretoken': token.value,
        })
        self.assertEqual(response.status_code, 403)

    def test_not_ok_without_token(self):
        """Test not ok without token"""
        response = self.client.post('/per-view')
        self.assertEqual(response.status_code, 403)



"""CSRF protection without cookies."""
from django.middleware import csrf as django_csrf
from .middlewares import CsrfMiddleware
# for compatibility with exists code:
from .context_processors import context_processor
from .decorators import anonymous_csrf, anonymous_csrf_exempt
from .utils import prep_key
from .conf import ANON_ALWAYS, ANON_COOKIE, ANON_TIMEOUT, PREFIX


def monkeypatch():
    from django.views.decorators import csrf as csrf_dec
    django_csrf.CsrfViewMiddleware = CsrfMiddleware
    csrf_dec.csrf_protect = csrf_dec.decorator_from_middleware(CsrfMiddleware)



from contextlib import contextmanager
import hashlib
from .models import Token
from . import conf


def prep_key(key):
    """
    In case a bogus request comes in with a large or wrongly formatted
    massive anoncsrf cookie value, memcache will raise a
    MemcachedKeyLengthError or MemcachedKeyCharacterError. We hash the
    key here in order to have a predictable length and character set.
    """
    prefixed = conf.PREFIX + key
    return hashlib.sha1(prefixed).hexdigest()


@contextmanager
def save_token(context):
    """Restore token value in context"""
    is_exists = 'csrf_token' in context
    token = context.get('csrf_token')
    yield
    if is_exists:
        context['csrf_token'] = token
    else:
        del context['csrf_token']


def get_token_for_request(request, view_name):
    """Get token for request"""
    if request.user.is_authenticated():
        token, _ = Token.objects.get_or_create(
            owner=request.user, for_view=view_name)
        return token



import os
from setuptools import setup, find_packages

ROOT = os.path.abspath(os.path.dirname(__file__))


setup(
    name='django-session-csrf-per-view',
    version='0.5.2',
    description='CSRF protection for Django without cookies.',
    long_description=open(os.path.join(ROOT, 'README.rst')).read(),
    author='Jeff Balogh',
    author_email='jbalogh@mozilla.com',
    maintainer='Vladimir Iakovlev',
    maintainer_email='nvbn.rm@gmail.com',
    url='http://github.com/mozilla/django-session-csrf',
    license='BSD',
    packages=find_packages(exclude=['example']),
    include_package_data=True,
    install_requires=['django'],
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Environment :: Web Environment :: Mozilla',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



from datetime import datetime
from django.db import models
from django.middleware.csrf import _get_new_csrf_key
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from . import conf


class TokenManager(models.Manager):
    """Token manager"""

    @property
    def _expiration_date(self):
        return datetime.now() - conf.CSRF_TOKEN_LIFETIME

    def get_expired(self):
        """Get expired tokens"""
        return self.filter(created__lt=self._expiration_date)

    def has_valid(self, owner, value, for_view=None):
        """Has valid token with user and value"""
        return self.filter(
            owner=owner, value=value, for_view=for_view,
            created__gte=self._expiration_date,
        ).exists()


class Token(models.Model):
    """Storage for csrf tokens"""
    value = models.CharField(max_length=32, verbose_name=_('token value'))
    owner = models.ForeignKey(User, verbose_name=_('owner'))
    created = models.DateTimeField(
        auto_now_add=True, verbose_name=_('created'),
    )
    for_view = models.CharField(
        null=True, blank=True,
        max_length=255, verbose_name=_('for view'),
    )

    objects = TokenManager()

    def save(self, *args, **kwargs):
        """Generate token on first save"""
        if not self.id:
            self.value = _get_new_csrf_key()
        return super(Token, self).save(*args, **kwargs)

    def __unicode__(self):
        return '{}:{}'.format(self.owner, self.created)
